@@ -1,4 +1,4 @@
-/* 
+/*
    Unix SMB/CIFS implementation.
    Samba system utilities
    Copyright (C) Andrew Tridgell 1992-1998
@@ -628,11 +628,14 @@ static bool set_process_capability(enum smbd_capability capability,
 	cap_set_flag(cap, CAP_INHERITABLE, num_cap_vals, cap_vals, CAP_CLEAR);
 
 	if (cap_set_proc(cap) == -1) {
-		DEBUG(0, ("set_process_capability: cap_set_proc failed: %s\n",
-			strerror(errno)));
+		DBG_ERR("%s capability %d: cap_set_proc failed: %s\n",
+			enable ? "adding" : "dropping",
+			capability, strerror(errno));
 		cap_free(cap);
 		return False;
 	}
+	DBG_INFO("%s capability %d\n",
+		 enable ? "added" : "dropped", capability);
 
 	cap_free(cap);
 	return True;
@@ -755,7 +758,7 @@ int getgroups_max(void)
 
 /**************************************************************************
  Wrap setgroups and getgroups for systems that declare getgroups() as
- returning an array of gid_t, but actuall return an array of int.
+ returning an array of gid_t, but actually return an array of int.
 ****************************************************************************/
 
 #if defined(HAVE_BROKEN_GETGROUPS)
@@ -781,9 +784,9 @@ static int sys_broken_getgroups(int setlen, gid_t *gidset)
 	 */
 
 	if(setlen < 0) {
-		errno = EINVAL; 
+		errno = EINVAL;
 		return -1;
-	} 
+	}
 
 	if((group_list = SMB_MALLOC_ARRAY(GID_T, setlen)) == NULL) {
 		DEBUG(0,("sys_getgroups: Malloc fail.\n"));
@@ -813,14 +816,14 @@ static int sys_broken_getgroups(int setlen, gid_t *gidset)
 static int sys_broken_setgroups(int setlen, gid_t *gidset)
 {
 	GID_T *group_list;
-	int i ; 
+	int i ;
 
 	if (setlen == 0)
 		return 0 ;
 
 	if (setlen < 0 || setlen > setgroups_max()) {
-		errno = EINVAL; 
-		return -1;   
+		errno = EINVAL;
+		return -1;
 	}
 
 	/*
@@ -830,11 +833,11 @@ static int sys_broken_setgroups(int setlen, gid_t *gidset)
 
 	if((group_list = SMB_MALLOC_ARRAY(GID_T, setlen)) == NULL) {
 		DEBUG(0,("sys_setgroups: Malloc fail.\n"));
-		return -1;    
+		return -1;
 	}
 
-	for(i = 0; i < setlen; i++) 
-		group_list[i] = (GID_T) gidset[i]; 
+	for(i = 0; i < setlen; i++)
+		group_list[i] = (GID_T) gidset[i];
 
 	if(samba_setgroups(setlen, group_list) != 0) {
 		int saved_errno = errno;
@@ -871,7 +874,7 @@ static int sys_bsd_setgroups(gid_t primary_gid, int setlen, const gid_t *gidset)
 	/* setgroups(2) will fail with EINVAL if we pass too many groups. */
 	max = setgroups_max();
 
-	/* No group list, just make sure we are setting the efective GID. */
+	/* No group list, just make sure we are setting the effective GID. */
 	if (setlen == 0) {
 		return samba_setgroups(1, &primary_gid);
 	}
@@ -1044,22 +1047,10 @@ int sys_get_number_of_cores(void)
 }
 #endif
 
-static struct proc_fd_pattern {
-	const char *pattern;
-	const char *test_path;
-} proc_fd_patterns[] = {
-	/* Linux */
-	{ "/proc/self/fd/%d", "/proc/self/fd/0" },
-	{ NULL, NULL },
-};
-
-static const char *proc_fd_pattern;
-
 bool sys_have_proc_fds(void)
 {
-	static bool checked;
-	static bool have_proc_fds;
-	struct proc_fd_pattern *p = NULL;
+	static bool checked = false;
+	static bool have_proc_fds = false;
 	struct stat sb;
 	int ret;
 
@@ -1067,42 +1058,19 @@ bool sys_have_proc_fds(void)
 		return have_proc_fds;
 	}
 
-	for (p = &proc_fd_patterns[0]; p->test_path != NULL; p++) {
-		ret = stat(p->test_path, &sb);
-		if (ret != 0) {
-			continue;
-		}
-		have_proc_fds = true;
-		proc_fd_pattern = p->pattern;
-		break;
-	}
-
+	ret = stat("/proc/self/fd/0", &sb);
+	have_proc_fds = (ret == 0);
 	checked = true;
+
 	return have_proc_fds;
 }
 
-const char *sys_proc_fd_path(int fd, char *buf, size_t bufsize)
+char *sys_proc_fd_path(int fd, struct sys_proc_fd_path_buf *buf)
 {
-	int written;
+	int written =
+		snprintf(buf->buf, sizeof(buf->buf), "/proc/self/fd/%d", fd);
 
-	if (!sys_have_proc_fds()) {
-		return NULL;
-	}
+	SMB_ASSERT(sys_have_proc_fds() && (written >= 0));
 
-#if defined(__clang__)
-#pragma clang diagnostic push
-#pragma clang diagnostic ignored "-Wformat-nonliteral"
-#endif
-	written = snprintf(buf,
-			   bufsize,
-			   proc_fd_pattern,
-			   fd);
-#if defined(__clang__)
-#pragma clang diagnostic pop
-#endif
-	if (written >= bufsize) {
-		return NULL;
-	}
-
-	return buf;
+	return buf->buf;
 }
