@@ -464,8 +464,10 @@ SMBC_getatr(SMBCCTX * context,
 	struct timespec access_time_ts = {0};
 	struct timespec write_time_ts = {0};
 	struct timespec change_time_ts = {0};
+	struct timespec w_time_ts = {0};
 	time_t write_time = 0;
 	SMB_INO_T ino = 0;
+	mode_t mode = S_IFREG;
 	struct cli_credentials *creds = NULL;
 	TALLOC_CTX *frame = talloc_stackframe();
 	NTSTATUS status;
@@ -506,6 +508,7 @@ SMBC_getatr(SMBCCTX * context,
 	}
 
 	if (!srv->no_pathinfo2) {
+		bool not_supported_error = false;
 		status = cli_qpathinfo2(targetcli,
 					targetpath,
 					&create_time_ts,
@@ -514,15 +517,26 @@ SMBC_getatr(SMBCCTX * context,
 					&change_time_ts,
 					&size,
 					&attr,
-					&ino);
+					&ino,
+					&mode);
 		if (NT_STATUS_IS_OK(status)) {
 			goto setup_stat;
 		}
+		if (NT_STATUS_EQUAL(status, NT_STATUS_INVALID_LEVEL) ||
+		    NT_STATUS_EQUAL(status, NT_STATUS_NOT_SUPPORTED)) {
+			not_supported_error = true;
+		}
+		if (!not_supported_error) {
+			/* "Normal error". Just return it to caller. */
+			TALLOC_FREE(frame);
+			return status;
+		}
         }
 
 	srv->no_pathinfo2 = True;
 
 	if (!srv->no_pathinfo3) {
+		bool not_supported_error = false;
 		status = cli_qpathinfo3(targetcli,
 					targetpath,
 					&create_time_ts,
@@ -535,6 +549,15 @@ SMBC_getatr(SMBCCTX * context,
 		if (NT_STATUS_IS_OK(status)) {
 			goto setup_stat;
 		}
+		if (NT_STATUS_EQUAL(status, NT_STATUS_INVALID_LEVEL) ||
+		    NT_STATUS_EQUAL(status, NT_STATUS_NOT_SUPPORTED)) {
+			not_supported_error = true;
+		}
+		if (!not_supported_error) {
+			/* "Normal error". Just return it to caller. */
+			TALLOC_FREE(frame);
+			return status;
+		}
         }
 
 	srv->no_pathinfo3 = True;
@@ -545,14 +568,11 @@ SMBC_getatr(SMBCCTX * context,
         }
 
 	status = cli_getatr(targetcli, targetpath, &attr, &size, &write_time);
-	if (NT_STATUS_IS_OK(status)) {
-		struct timespec w_time_ts =
-			convert_time_t_to_timespec(write_time);
-
-		access_time_ts = change_time_ts = write_time_ts = w_time_ts;
-
-		goto setup_stat;
+	if (!NT_STATUS_IS_OK(status)) {
+		goto all_failed;
 	}
+	w_time_ts = convert_time_t_to_timespec(write_time);
+	access_time_ts = change_time_ts = write_time_ts = w_time_ts;
 
 setup_stat:
 	setup_stat(sb,
@@ -565,6 +585,10 @@ setup_stat:
 		   change_time_ts,
 		   write_time_ts);
 
+	if ((context->internal->posix_extensions) && (mode != S_IFREG)) {
+		sb->st_mode = (sb->st_mode & ~S_IFMT) | mode;
+	}
+
 	TALLOC_FREE(frame);
 	return NT_STATUS_OK;
 
@@ -573,7 +597,7 @@ all_failed:
 	srv->no_pathinfo3 = False;
 
 	TALLOC_FREE(frame);
-	return NT_STATUS_ACCESS_DENIED;
+	return status;
 }
 
 /*
