@@ -1556,12 +1556,11 @@ static int shadow_copy2_openat(vfs_handle_struct *handle,
 	struct smb_filename *smb_fname = NULL;
 	time_t timestamp = 0;
 	char *stripped = NULL;
-	bool is_converted = false;
 	int saved_errno = 0;
 	int ret;
 	bool ok;
 
-	if (how.resolve != 0) {
+	if ((how.resolve & ~VFS_OPEN_HOW_WITH_BACKUP_INTENT) != 0) {
 		errno = ENOSYS;
 		return -1;
 	}
@@ -1573,26 +1572,18 @@ static int shadow_copy2_openat(vfs_handle_struct *handle,
 		return -1;
 	}
 
-	ok = shadow_copy2_strip_snapshot_converted(talloc_tos(),
-						   handle,
-						   smb_fname,
-						   &timestamp,
-						   &stripped,
-						   &is_converted);
+	ok = shadow_copy2_strip_snapshot(talloc_tos(),
+					 handle,
+					 smb_fname,
+					 &timestamp,
+					 &stripped);
 	if (!ok) {
+		TALLOC_FREE(smb_fname);
 		return -1;
 	}
 	if (timestamp == 0) {
-		if (is_converted) {
-			/*
-			 * Just pave over the user requested mode and use
-			 * O_RDONLY. Later attempts by the client to write on
-			 * the handle will fail in the pwrite() syscall with
-			 * EINVAL which we carefully map to EROFS. In sum, this
-			 * matches Windows behaviour.
-			 */
-			how.flags &= ~(O_WRONLY | O_RDWR | O_CREAT);
-		}
+		TALLOC_FREE(stripped);
+		TALLOC_FREE(smb_fname);
 		return SMB_VFS_NEXT_OPENAT(handle,
 					   dirfsp,
 					   smb_fname_in,
@@ -1613,14 +1604,6 @@ static int shadow_copy2_openat(vfs_handle_struct *handle,
 	}
 	TALLOC_FREE(stripped);
 
-	/*
-	 * Just pave over the user requested mode and use O_RDONLY. Later
-	 * attempts by the client to write on the handle will fail in the
-	 * pwrite() syscall with EINVAL which we carefully map to EROFS. In sum,
-	 * this matches Windows behaviour.
-	 */
-	how.flags &= ~(O_WRONLY | O_RDWR | O_CREAT);
-
 	ret = SMB_VFS_NEXT_OPENAT(handle,
 				  dirfsp,
 				  smb_fname,
@@ -2386,6 +2369,7 @@ static int shadow_copy2_mkdirat(vfs_handle_struct *handle,
 					full_fname,
 					&timestamp,
 					NULL)) {
+		TALLOC_FREE(full_fname);
 		return -1;
 	}
 	TALLOC_FREE(full_fname);
@@ -2540,100 +2524,6 @@ static NTSTATUS shadow_copy2_read_dfs_pathat(struct vfs_handle_struct *handle,
 	return status;
 }
 
-static NTSTATUS shadow_copy2_get_real_filename_at(
-	struct vfs_handle_struct *handle,
-	struct files_struct *dirfsp,
-	const char *name,
-	TALLOC_CTX *mem_ctx,
-	char **found_name)
-{
-	struct shadow_copy2_private *priv = NULL;
-	time_t timestamp = 0;
-	char *stripped = NULL;
-	char *conv;
-	struct smb_filename *conv_fname = NULL;
-	NTSTATUS status;
-	bool ok;
-
-	SMB_VFS_HANDLE_GET_DATA(handle, priv, struct shadow_copy2_private,
-				return NT_STATUS_INTERNAL_ERROR);
-
-	DBG_DEBUG("Path=[%s] name=[%s]\n", fsp_str_dbg(dirfsp), name);
-
-	ok = shadow_copy2_strip_snapshot(
-		talloc_tos(), handle, dirfsp->fsp_name, &timestamp, &stripped);
-	if (!ok) {
-		status = map_nt_error_from_unix(errno);
-		DEBUG(10, ("shadow_copy2_strip_snapshot failed\n"));
-		return status;
-	}
-	if (timestamp == 0) {
-		DEBUG(10, ("timestamp == 0\n"));
-		return SMB_VFS_NEXT_GET_REAL_FILENAME_AT(
-			handle, dirfsp, name, mem_ctx, found_name);
-	}
-
-	/*
-	 * Note that stripped may be an empty string "" if path was ".". As
-	 * shadow_copy2_convert() combines "" with the shadow-copy tree connect
-	 * root fullpath and get_real_filename_full_scan() has an explicit check
-	 * for "" this works.
-	 */
-	DBG_DEBUG("stripped [%s]\n", stripped);
-
-	conv = shadow_copy2_convert(talloc_tos(), handle, stripped, timestamp);
-	if (conv == NULL) {
-		status = map_nt_error_from_unix(errno);
-		DBG_DEBUG("shadow_copy2_convert [%s] failed: %s\n",
-			  stripped,
-			  strerror(errno));
-		return status;
-	}
-
-	status = synthetic_pathref(
-		talloc_tos(),
-		dirfsp->conn->cwd_fsp,
-		conv,
-		NULL,
-		NULL,
-		0,
-		0,
-		&conv_fname);
-	if (!NT_STATUS_IS_OK(status)) {
-		return status;
-	}
-
-	DEBUG(10, ("Calling NEXT_GET_REAL_FILE_NAME for conv=[%s], "
-		   "name=[%s]\n", conv, name));
-	status = SMB_VFS_NEXT_GET_REAL_FILENAME_AT(
-		handle, conv_fname->fsp, name, mem_ctx, found_name);
-	DEBUG(10, ("NEXT_REAL_FILE_NAME returned %s\n", nt_errstr(status)));
-	if (NT_STATUS_IS_OK(status)) {
-		TALLOC_FREE(conv_fname);
-		return NT_STATUS_OK;
-	}
-	if (!NT_STATUS_EQUAL(status, NT_STATUS_NOT_SUPPORTED)) {
-		TALLOC_FREE(conv_fname);
-		TALLOC_FREE(conv);
-		return NT_STATUS_NOT_SUPPORTED;
-	}
-
-	status = get_real_filename_full_scan_at(
-		conv_fname->fsp, name, false, mem_ctx, found_name);
-	TALLOC_FREE(conv_fname);
-	if (!NT_STATUS_IS_OK(status)) {
-		DBG_DEBUG("Scan [%s] for [%s] failed\n",
-			  conv, name);
-		return status;
-	}
-
-	DBG_DEBUG("Scan [%s] for [%s] returned [%s]\n",
-		  conv, name, *found_name);
-
-	TALLOC_FREE(conv);
-	return NT_STATUS_OK;
-}
-
 static const char *shadow_copy2_connectpath(
 	struct vfs_handle_struct *handle,
 	const struct files_struct *dirfsp,
@@ -3398,7 +3288,6 @@ static struct vfs_fn_pointers vfs_shadow_copy2_fns = {
 	.mkdirat_fn = shadow_copy2_mkdirat,
 	.fsetxattr_fn = shadow_copy2_fsetxattr,
 	.fchflags_fn = shadow_copy2_fchflags,
-	.get_real_filename_at_fn = shadow_copy2_get_real_filename_at,
 	.pwrite_fn = shadow_copy2_pwrite,
 	.pwrite_send_fn = shadow_copy2_pwrite_send,
 	.pwrite_recv_fn = shadow_copy2_pwrite_recv,
