@@ -34,6 +34,7 @@
 #include "messages.h"
 #include "../librpc/gen_ndr/open_files.h"
 #include "lib/util/tevent_ntstatus.h"
+#include "source3/smbd/dir.h"
 
 /****************************************************************************
  Run a file if it is a magic script.
@@ -241,8 +242,7 @@ static bool has_other_nonposix_opens_fn(
 	if (e->name_hash != fsp->name_hash) {
 		return false;
 	}
-	if ((fsp->posix_flags & FSP_POSIX_FLAGS_OPEN) &&
-	    (e->flags & SHARE_MODE_FLAG_POSIX_OPEN)) {
+	if (e->flags & SHARE_MODE_FLAG_POSIX_OPEN) {
 		return false;
 	}
 	if (e->share_file_id == fh_get_gen_id(fsp->fh)) {
@@ -603,6 +603,7 @@ static NTSTATUS close_remove_share_mode(files_struct *fsp,
  	 */
 
 	fsp->fsp_flags.delete_on_close = false;
+	fsp->fsp_flags.fstat_before_close = false;
 	lck_state.reset_delete_on_close = true;
 
  done:
@@ -1463,9 +1464,9 @@ static NTSTATUS rmdir_internals(TALLOC_CTX *ctx, struct files_struct *fsp)
 }
 
 /****************************************************************************
- Close a directory opened by an NT SMB call. 
+ Close a directory opened by an NT SMB call.
 ****************************************************************************/
-  
+
 static NTSTATUS close_directory(struct smb_request *req, files_struct *fsp,
 				enum file_close_type close_type)
 {
@@ -1505,7 +1506,8 @@ static NTSTATUS close_directory(struct smb_request *req, files_struct *fsp,
 	if (!NT_STATUS_IS_OK(status)) {
 		DBG_ERR("share_mode_entry_prepare_lock_del() failed for %s - %s\n",
 			fsp_str_dbg(fsp), nt_errstr(status));
-		return status;
+		log_stack_trace();
+		goto close_fd;
 	}
 
 	/*
@@ -1594,6 +1596,7 @@ done:
 
 	remove_pending_change_notify_requests_by_fid(fsp, notify_status);
 
+close_fd:
 	status1 = fd_close(fsp);
 
 	if (!NT_STATUS_IS_OK(status1)) {
@@ -1611,7 +1614,7 @@ done:
 /****************************************************************************
  Rundown all SMB-related dependencies of a files struct
 ****************************************************************************/
-  
+
 NTSTATUS close_file_smb(struct smb_request *req,
 			struct files_struct *fsp,
 			enum file_close_type close_type)
@@ -1630,6 +1633,14 @@ NTSTATUS close_file_smb(struct smb_request *req,
 	SMB_ASSERT(fsp->stream_fsp == NULL);
 
 	if (fsp->fake_file_handle != NULL) {
+		/*
+		 * Named pipes are opened as fake files and
+		 * can have pending aio requests. Ensure
+		 * we clear out all pending aio on force
+		 * shutdown of named pipes also.
+		 * BUG: https://bugzilla.samba.org/show_bug.cgi?id=15423
+		 */
+		assert_no_pending_aio(fsp, close_type);
 		status = close_fake_file(req, fsp);
 	} else if (fsp->print_file != NULL) {
 		/* FIXME: return spool errors */
