@@ -1,4 +1,4 @@
-/* 
+/*
    Unix SMB/CIFS implementation.
    Samba utility functions
 
@@ -102,8 +102,8 @@ int dsdb_module_search_dn(struct ldb_module *module,
 	if (res->count != 1) {
 		/* we may be reading a DB that does not have the 'check base on search' option... */
 		ret = LDB_ERR_NO_SUCH_OBJECT;
-		ldb_asprintf_errstring(ldb_module_get_ctx(module), 
-				       "dsdb_module_search_dn: did not find base dn %s (%d results)", 
+		ldb_asprintf_errstring(ldb_module_get_ctx(module),
+				       "dsdb_module_search_dn: did not find base dn %s (%d results)",
 				       ldb_dn_get_linearized(basedn), res->count);
 	} else {
 		*_res = talloc_steal(mem_ctx, res);
@@ -255,14 +255,16 @@ int dsdb_module_search(struct ldb_module *module,
 }
 
 /*
-  find a DN given a GUID. This searches across all partitions
+  find an object given a GUID. This searches across all partitions
  */
-int dsdb_module_dn_by_guid(struct ldb_module *module, TALLOC_CTX *mem_ctx,
-			   const struct GUID *guid, struct ldb_dn **dn,
-			   struct ldb_request *parent)
+int dsdb_module_obj_by_guid(struct ldb_module *module,
+			    TALLOC_CTX *mem_ctx,
+			    struct ldb_message **_msg,
+			    const struct GUID *guid,
+			    const char * const *attrs,
+			    struct ldb_request *parent)
 {
 	struct ldb_result *res;
-	const char *attrs[] = { NULL };
 	TALLOC_CTX *tmp_ctx = talloc_new(mem_ctx);
 	int ret;
 
@@ -289,7 +291,36 @@ int dsdb_module_dn_by_guid(struct ldb_module *module, TALLOC_CTX *mem_ctx,
 		return LDB_ERR_OPERATIONS_ERROR;
 	}
 
-	*dn = talloc_steal(mem_ctx, res->msgs[0]->dn);
+	*_msg = talloc_steal(mem_ctx, res->msgs[0]);
+
+	talloc_free(tmp_ctx);
+	return LDB_SUCCESS;
+}
+
+/*
+  find a DN given a GUID. This searches across all partitions
+ */
+int dsdb_module_dn_by_guid(struct ldb_module *module, TALLOC_CTX *mem_ctx,
+			   const struct GUID *guid, struct ldb_dn **dn,
+			   struct ldb_request *parent)
+{
+	struct ldb_message *msg = NULL;
+	static const char * const attrs[] = { NULL };
+	TALLOC_CTX *tmp_ctx = talloc_new(mem_ctx);
+	int ret;
+
+	ret = dsdb_module_obj_by_guid(module,
+				      tmp_ctx,
+				      &msg,
+				      guid,
+				      attrs,
+				      parent);
+	if (ret != LDB_SUCCESS) {
+		talloc_free(tmp_ctx);
+		return ret;
+	}
+
+	*dn = talloc_steal(mem_ctx, msg->dn);
 
 	talloc_free(tmp_ctx);
 	return LDB_SUCCESS;
@@ -301,7 +332,7 @@ int dsdb_module_dn_by_guid(struct ldb_module *module, TALLOC_CTX *mem_ctx,
 int dsdb_module_guid_by_dn(struct ldb_module *module, struct ldb_dn *dn, struct GUID *guid,
 			   struct ldb_request *parent)
 {
-	const char *attrs[] = { NULL };
+	static const char * const attrs[] = { NULL };
 	struct ldb_result *res;
 	TALLOC_CTX *tmp_ctx = talloc_new(module);
 	int ret;
@@ -690,7 +721,7 @@ int dsdb_check_samba_compatible_feature(struct ldb_module *module,
 {
 	struct ldb_context *ldb = ldb_module_get_ctx(module);
 	struct ldb_result *res;
-	static const char *samba_dsdb_attrs[] = {
+	static const char * const samba_dsdb_attrs[] = {
 		SAMBA_COMPATIBLE_FEATURES_ATTR,
 		NULL
 	};
@@ -735,7 +766,7 @@ int dsdb_check_samba_compatible_feature(struct ldb_module *module,
 
   Note that features can be marked as enabled in more than one
   place. For example, the recyclebin feature is marked as enabled both
-  on the CN=Partitions,CN=Configurration object and on the NTDS DN of
+  on the CN=Partitions,CN=Configuration object and on the NTDS DN of
   each DC in the forest. It seems likely that it is the job of the KCC
   to propagate between the two
  */
@@ -746,7 +777,7 @@ int dsdb_check_optional_feature(struct ldb_module *module, struct GUID op_featur
 	struct ldb_result *res;
 	struct ldb_dn *search_dn;
 	struct GUID search_guid;
-	const char *attrs[] = {"msDS-EnabledFeature", NULL};
+	static const char * const attrs[] = {"msDS-EnabledFeature", NULL};
 	int ret;
 	unsigned int i;
 	struct ldb_message_element *el;
@@ -771,7 +802,7 @@ int dsdb_check_optional_feature(struct ldb_module *module, struct GUID op_featur
 		return LDB_ERR_NO_SUCH_OBJECT;
 	}
 	if (res->msgs[0]->num_elements > 0) {
-		const char *attrs2[] = {"msDS-OptionalFeatureGUID", NULL};
+		static const char * const attrs2[] = {"msDS-OptionalFeatureGUID", NULL};
 
 		el = ldb_msg_find_element(res->msgs[0],"msDS-EnabledFeature");
 
@@ -1349,7 +1380,7 @@ const struct ldb_val *dsdb_module_find_dsheuristics(struct ldb_module *module,
 	int ret;
 	struct ldb_dn *new_dn;
 	struct ldb_context *ldb = ldb_module_get_ctx(module);
-	static const char *attrs[] = { "dSHeuristics", NULL };
+	static const char * const attrs[] = { "dSHeuristics", NULL };
 	struct ldb_result *res;
 
 	new_dn = ldb_dn_copy(mem_ctx, ldb_get_config_basedn(ldb));
@@ -1433,6 +1464,46 @@ bool dsdb_do_list_object(struct ldb_module *module,
 	return result;
 }
 
+bool dsdb_attribute_authz_on_ldap_add(struct ldb_module *module,
+				      TALLOC_CTX *mem_ctx,
+				      struct ldb_request *parent)
+{
+	TALLOC_CTX *tmp_ctx = talloc_new(mem_ctx);
+	bool result = false;
+	const struct ldb_val *hr_val = dsdb_module_find_dsheuristics(module,
+								     tmp_ctx,
+								     parent);
+	if (hr_val != NULL && hr_val->length >= DS_HR_ATTR_AUTHZ_ON_LDAP_ADD) {
+		uint8_t val = hr_val->data[DS_HR_ATTR_AUTHZ_ON_LDAP_ADD - 1];
+		if (val != '0' && val != '2') {
+			result = true;
+		}
+	}
+
+	talloc_free(tmp_ctx);
+	return result;
+}
+
+bool dsdb_block_owner_implicit_rights(struct ldb_module *module,
+				      TALLOC_CTX *mem_ctx,
+				      struct ldb_request *parent)
+{
+	TALLOC_CTX *tmp_ctx = talloc_new(mem_ctx);
+	bool result = false;
+	const struct ldb_val *hr_val = dsdb_module_find_dsheuristics(module,
+								     tmp_ctx,
+								     parent);
+	if (hr_val != NULL && hr_val->length >= DS_HR_BLOCK_OWNER_IMPLICIT_RIGHTS) {
+		uint8_t val = hr_val->data[DS_HR_BLOCK_OWNER_IMPLICIT_RIGHTS - 1];
+		if (val != '0' && val != '2') {
+			result = true;
+		}
+	}
+
+	talloc_free(tmp_ctx);
+	return result;
+}
+
 /*
   show the chain of requests, useful for debugging async requests
  */
@@ -1491,7 +1562,7 @@ int dsdb_get_expected_new_values(TALLOC_CTX *mem_ctx,
 			el_count++;
 			tmp = val_count + msg->elements[i].num_values;
 			if (unlikely(tmp < val_count)) {
-				DBG_ERR("too many values for one element!");
+				DBG_ERR("too many values for one element!\n");
 				return LDB_ERR_OPERATIONS_ERROR;
 			}
 			val_count = tmp;
@@ -1757,8 +1828,8 @@ bool dsdb_is_subclass_of(const struct dsdb_schema *schema,
 }
 
 /* Fix the DN so that the relative attribute names are in upper case so that the DN:
-   cn=Adminstrator,cn=users,dc=samba,dc=example,dc=com becomes
-   CN=Adminstrator,CN=users,DC=samba,DC=example,DC=com
+   cn=Administrator,cn=users,dc=samba,dc=example,dc=com becomes
+   CN=Administrator,CN=users,DC=samba,DC=example,DC=com
 */
 int dsdb_fix_dn_rdncase(struct ldb_context *ldb, struct ldb_dn *dn)
 {
@@ -1791,7 +1862,7 @@ int dsdb_fix_dn_rdncase(struct ldb_context *ldb, struct ldb_dn *dn)
  * @param ldb	ldb context
  * @param schema cached schema for ldb. We may get it, but it is very time consuming.
  * 			Hence leave the responsibility to the caller.
- * @param obj	AD object to determint objectCategory for
+ * @param obj	AD object to determine objectCategory for
  * @param mem_ctx Memory context - usually it is obj actually
  * @param pobjectcategory location to store found objectCategory
  *
