@@ -1,5 +1,5 @@
 /* 
-   Unix SMB/CIFS mplementation.
+   Unix SMB/CIFS Implementation.
    DSDB schema header
    
    Copyright (C) Stefan Metzmacher <metze@samba.org> 2006-2007
@@ -27,6 +27,9 @@
 #include "lib/util/tsort.h"
 #include "util/dlinklist.h"
 
+#undef strcasecmp
+#undef strncasecmp
+
 static const char **dsdb_full_attribute_list_internal(TALLOC_CTX *mem_ctx, 
 						      const struct dsdb_schema *schema, 
 						      const char **class_list,
@@ -123,6 +126,16 @@ const struct dsdb_attribute *dsdb_attribute_by_linkID(const struct dsdb_schema *
 	return c;
 }
 
+const struct dsdb_attribute *dsdb_attribute_by_cn_ldb_val(const struct dsdb_schema *schema,
+							  const struct ldb_val *cn)
+{
+	struct dsdb_attribute *c;
+
+	BINARY_ARRAY_SEARCH_P(schema->attributes_by_cn,
+			      schema->num_attributes, cn, cn, strcasecmp_with_ldb_val, c);
+	return c;
+}
+
 const struct dsdb_class *dsdb_class_by_governsID_id(const struct dsdb_schema *schema,
 						    uint32_t id)
 {
@@ -254,7 +267,7 @@ const char **merge_attr_list(TALLOC_CTX *mem_ctx,
 
 /*
   Return a merged list of the attributes of exactly one class (not
-  considering subclasses, auxillary classes etc)
+  considering subclasses, auxiliary classes etc)
 */
 
 const char **dsdb_attribute_list(TALLOC_CTX *mem_ctx, const struct dsdb_class *sclass, enum dsdb_attr_list_query query)
@@ -483,7 +496,7 @@ int dsdb_sort_objectClass_attr(struct ldb_context *ldb,
 	 * * unsorted:     The remaining objectClasses
 	 *
 	 * The process is a matter of filtering objectClasses up from
-	 * unsorted into sorted.  Order is irrelevent in the later 3 'bins'.
+	 * unsorted into sorted.  Order is irrelevant in the later 3 'bins'.
 	 *
 	 * We start with 'top' (found and promoted to parent_class
 	 * initially).  Then we find (in unsorted) all the direct
