@@ -2,7 +2,7 @@
    Unix SMB/CIFS implementation.
    test suite for iremotewinspool rpc operations
 
-   Copyright (C) Guenther Deschner 2013
+   Copyright (C) Guenther Deschner 2013,2023
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,218 +25,7 @@
 #include "librpc/gen_ndr/ndr_spoolss_c.h"
 #include "torture/rpc/torture_rpc.h"
 #include "libcli/registry/util_reg.h"
-
-struct test_iremotewinspool_context {
-	struct GUID object_uuid;
-	struct dcerpc_pipe *iremotewinspool_pipe;
-	struct policy_handle server_handle;
-	const char *environment;
-};
-
-enum client_os_version
-{
-	WIN_2000,
-	WIN_VISTA,
-	WIN_SERVER_2008,
-	WIN_7,
-	WIN_SERVER_2008R2,
-	WIN_8,
-	WIN_SERVER_2012,
-	WIN_10,
-	WIN_SERVER_2016
-};
-
-static struct spoolss_UserLevel1 test_get_client_info(struct torture_context *tctx,
-						      enum client_os_version os,
-						      enum spoolss_MajorVersion major_number,
-						      enum spoolss_MinorVersion minor_number)
-{
-	struct spoolss_UserLevel1 level1;
-
-	level1.size	= 28;
-	level1.client	= talloc_asprintf(tctx, "\\\\%s", "mthelena");
-	level1.user	= "GD";
-	level1.processor = PROCESSOR_ARCHITECTURE_AMD64;
-	level1.major	= major_number;
-	level1.minor	= minor_number;
-
-	switch (os) {
-		case WIN_SERVER_2016:
-		case WIN_10:
-			level1.build = 10586;
-			break;
-		case WIN_SERVER_2012:
-		case WIN_8:
-			level1.build = 9200;
-			break;
-		case WIN_SERVER_2008R2:
-		case WIN_7:
-			level1.build = 7007;
-			break;
-		case WIN_SERVER_2008:
-		case WIN_VISTA:
-			level1.build = 6000;
-			break;
-		case WIN_2000:
-			level1.build = 1382;
-			break;
-		default:
-			level1.build = 7007;
-	}
-
-	return level1;
-}
-
-static bool test_AsyncOpenPrinter_byprinter(struct torture_context *tctx,
-					    struct test_iremotewinspool_context *ctx,
-					    struct dcerpc_pipe *p,
-					    const char *printer_name,
-					    struct spoolss_UserLevel1 cinfo,
-					    struct policy_handle *handle)
-{
-	struct dcerpc_binding_handle *b = p->binding_handle;
-	struct spoolss_DevmodeContainer devmode_ctr;
-	struct spoolss_UserLevelCtr client_info_ctr;
-	uint32_t access_mask = SERVER_ALL_ACCESS;
-	struct winspool_AsyncOpenPrinter r;
-
-	ZERO_STRUCT(devmode_ctr);
-
-	client_info_ctr.level = 1;
-	client_info_ctr.user_info.level1 = &cinfo;
-
-	r.in.pPrinterName	= printer_name;
-	r.in.pDatatype		= NULL;
-	r.in.pDevModeContainer	= &devmode_ctr;
-	r.in.AccessRequired	= access_mask;
-	r.in.pClientInfo	= &client_info_ctr;
-	r.out.pHandle		= handle;
-
-	torture_assert_ntstatus_ok(tctx,
-		dcerpc_winspool_AsyncOpenPrinter_r(b, tctx, &r),
-		"AsyncOpenPrinter failed");
-	torture_assert_werr_ok(tctx, r.out.result,
-		"AsyncOpenPrinter failed");
-
-	return true;
-}
-
-static bool test_AsyncClosePrinter_byhandle(struct torture_context *tctx,
-					    struct test_iremotewinspool_context *ctx,
-					    struct dcerpc_pipe *p,
-					    struct policy_handle *handle)
-{
-	struct dcerpc_binding_handle *b = p->binding_handle;
-
-	struct winspool_AsyncClosePrinter r;
-
-	r.in.phPrinter = handle;
-	r.out.phPrinter = handle;
-
-	torture_assert_ntstatus_ok(tctx,
-		dcerpc_winspool_AsyncClosePrinter_r(b, tctx, &r),
-		"AsyncClosePrinter failed");
-	torture_assert_werr_ok(tctx, r.out.result,
-		"AsyncClosePrinter failed");
-
-	return true;
-}
-
-static bool test_AsyncGetPrinterData_checktype(struct torture_context *tctx,
-					       struct dcerpc_binding_handle *b,
-					       struct policy_handle *handle,
-					       const char *value_name,
-					       enum winreg_Type *expected_type,
-					       enum winreg_Type *type_p,
-					       uint8_t **data_p,
-					       uint32_t *needed_p)
-{
-	struct winspool_AsyncGetPrinterData r;
-	enum winreg_Type type;
-	uint32_t needed;
-
-	r.in.hPrinter = *handle;
-	r.in.pValueName = value_name;
-	r.in.nSize = 0;
-	r.out.pType = &type;
-	r.out.pData = talloc_zero_array(tctx, uint8_t, r.in.nSize);
-	r.out.pcbNeeded = &needed;
-
-	torture_comment(tctx, "Testing AsyncGetPrinterData(%s)\n",
-		r.in.pValueName);
-
-	torture_assert_ntstatus_ok(tctx,
-		dcerpc_winspool_AsyncGetPrinterData_r(b, tctx, &r),
-		"AsyncGetPrinterData failed");
-
-	if (W_ERROR_EQUAL(r.out.result, WERR_MORE_DATA)) {
-		if (expected_type) {
-			torture_assert_int_equal(tctx, type, *expected_type, "unexpected type");
-		}
-		r.in.nSize = needed;
-		r.out.pData = talloc_zero_array(tctx, uint8_t, r.in.nSize);
-
-		torture_assert_ntstatus_ok(tctx,
-			dcerpc_winspool_AsyncGetPrinterData_r(b, tctx, &r),
-			"AsyncGetPrinterData failed");
-	}
-
-	torture_assert_werr_ok(tctx, r.out.result,
-		"AsyncGetPrinterData failed");
-
-	if (type_p) {
-		*type_p = type;
-	}
-
-	if (data_p) {
-		*data_p = r.out.pData;
-	}
-
-	if (needed_p) {
-		*needed_p = needed;
-	}
-
-	return true;
-}
-
-static bool test_AsyncGetPrinterData_args(struct torture_context *tctx,
-					  struct dcerpc_binding_handle *b,
-					  struct policy_handle *handle,
-					  const char *value_name,
-					  enum winreg_Type *type_p,
-					  uint8_t **data_p,
-					  uint32_t *needed_p)
-{
-	return test_AsyncGetPrinterData_checktype(tctx, b, handle,
-						  value_name,
-						  NULL,
-						  type_p, data_p, needed_p);
-}
-
-static bool test_get_environment(struct torture_context *tctx,
-				 struct dcerpc_binding_handle *b,
-				 struct policy_handle *handle,
-				 const char **architecture)
-{
-	DATA_BLOB blob;
-	enum winreg_Type type;
-	uint8_t *data;
-	uint32_t needed;
-
-	torture_assert(tctx,
-		test_AsyncGetPrinterData_args(tctx, b, handle, "Architecture", &type, &data, &needed),
-		"failed to get Architecture");
-
-	torture_assert_int_equal(tctx, type, REG_SZ, "unexpected type");
-
-	blob = data_blob_const(data, needed);
-
-	torture_assert(tctx,
-		pull_reg_sz(tctx, &blob, architecture),
-		"failed to pull environment");
-
-	return true;
-}
+#include "torture/rpc/iremotewinspool_common.h"
 
 static bool torture_rpc_iremotewinspool_setup_common(struct torture_context *tctx,
 						     struct test_iremotewinspool_context *t)
@@ -263,7 +52,7 @@ static bool torture_rpc_iremotewinspool_setup_common(struct torture_context *tct
 
 	printer_name = talloc_asprintf(tctx, "\\\\%s", dcerpc_server_name(t->iremotewinspool_pipe));
 
-	client_info = test_get_client_info(tctx, WIN_7, 6, 1);
+	client_info = test_get_client_info(tctx, WIN_7, 6, 1, "testclient_machine", "testclient_user");
 
 	torture_assert(tctx,
 		test_AsyncOpenPrinter_byprinter(tctx, t,
@@ -323,7 +112,7 @@ static bool test_AsyncClosePrinter(struct torture_context *tctx,
 
 	printer_name = talloc_asprintf(tctx, "\\\\%s", dcerpc_server_name(p));
 
-	client_info = test_get_client_info(tctx, WIN_7, 6, 1);
+	client_info = test_get_client_info(tctx, WIN_7, 6, 1, "testclient_machine", "testclient_user");
 
 	torture_assert(tctx,
 		test_AsyncOpenPrinter_byprinter(tctx, ctx, p, printer_name, client_info, &handle),
@@ -349,7 +138,7 @@ static bool test_AsyncOpenPrinter(struct torture_context *tctx,
 
 	printer_name = talloc_asprintf(tctx, "\\\\%s", dcerpc_server_name(p));
 
-	client_info = test_get_client_info(tctx, WIN_7, 6, 1);
+	client_info = test_get_client_info(tctx, WIN_7, 6, 1, "testclient_machine", "testclient_user");
 
 	torture_assert(tctx,
 		test_AsyncOpenPrinter_byprinter(tctx, ctx, p, printer_name, client_info, &handle),
@@ -389,7 +178,8 @@ static bool test_AsyncOpenPrinterValidateBuildNumber(struct torture_context *tct
 	torture_assert_not_null(tctx, printer_name, "Cannot allocate memory");
 
 	/* fail with Windows 2000 build number */
-	client_info = test_get_client_info(tctx, WIN_2000, 3, SPOOLSS_MINOR_VERSION_0);
+	client_info = test_get_client_info(tctx, WIN_2000, 3, SPOOLSS_MINOR_VERSION_0,
+					   "testclient_machine", "testclient_user");
 
 	ZERO_STRUCT(devmode_ctr);
 
@@ -408,7 +198,8 @@ static bool test_AsyncOpenPrinterValidateBuildNumber(struct torture_context *tct
 		"AsyncOpenPrinter should have failed");
 
 	/* succeed with Windows 7 build number */
-	client_info = test_get_client_info(tctx, WIN_7, 6, 1);
+	client_info = test_get_client_info(tctx, WIN_7, 6, 1,
+					   "testclient_machine", "testclient_user");
 	client_info_ctr.user_info.level1 = &client_info;
 	r.in.pClientInfo	= &client_info_ctr;
 
@@ -610,8 +401,8 @@ static bool test_AsyncUploadPrinterDriverPackage(struct torture_context *tctx,
 		"AsyncUploadPrinterDriverPackage failed");
 
 	pcchDestInfPath = 260;
-	r.in.pszDestInfPath = talloc_zero_array(tctx, uint16_t, pcchDestInfPath);
-	r.out.pszDestInfPath = talloc_zero_array(tctx, uint16_t, pcchDestInfPath);
+	r.in.pszDestInfPath = talloc_zero(tctx, const char);
+	r.out.pszDestInfPath = talloc_zero(tctx, const char);
 
 	torture_assert_ntstatus_ok(tctx,
 		dcerpc_winspool_AsyncUploadPrinterDriverPackage_r(b, tctx, &r),
@@ -1011,7 +802,7 @@ static bool test_OpenPrinter(struct torture_context *tctx,
 
 	printer_name = talloc_asprintf(tctx, "\\\\%s", dcerpc_server_name(p));
 
-	client_info = test_get_client_info(tctx, WIN_7, 6, 1);
+	client_info = test_get_client_info(tctx, WIN_7, 6, 1, "testclient_machine", "testclient_user");
 
 	torture_assert(tctx,
 		test_AsyncOpenPrinter_byprinter(tctx, ctx, p, printer_name, client_info, &handle),
@@ -1031,6 +822,237 @@ static bool test_OpenPrinter(struct torture_context *tctx,
 	return true;
 }
 
+static bool test_object_one_uuid(struct torture_context *tctx,
+				 const struct GUID *object_uuid,
+				 NTSTATUS expected_status,
+				 uint32_t expected_fault_code)
+{
+	const char *printer_name;
+	struct spoolss_UserLevel1 client_info;
+	struct dcerpc_binding *binding;
+	struct dcerpc_pipe *p;
+	struct policy_handle server_handle;
+
+	torture_comment(tctx, "Testing with object_uuid: %s\n",
+			GUID_string(tctx, object_uuid));
+
+	torture_assert_ntstatus_ok(tctx,
+		torture_rpc_binding(tctx, &binding),
+		"failed to retrieve torture binding");
+
+	if (object_uuid) {
+		torture_assert_ntstatus_ok(tctx,
+			dcerpc_binding_set_object(binding, *object_uuid),
+			"failed to set object_uuid");
+	}
+
+	torture_assert_ntstatus_ok(tctx,
+		torture_rpc_connection_with_binding(tctx, binding, &p,
+			&ndr_table_iremotewinspool),
+		"Error connecting to server");
+
+	printer_name = talloc_asprintf(tctx, "\\\\%s", dcerpc_server_name(p));
+	torture_assert(tctx, printer_name, "out of memory");
+
+	client_info = test_get_client_info(tctx,
+		WIN_7, 6, 1, "testclient_machine", "testclient_user");
+
+	torture_assert(tctx,
+		test_AsyncOpenPrinter_byprinter_expect(tctx, NULL,
+						       p,
+						       printer_name,
+						       SEC_FLAG_MAXIMUM_ALLOWED,
+						       client_info,
+						       expected_status,
+						       WERR_OK,
+						       expected_fault_code,
+						       &server_handle),
+						"failed to open printserver");
+	if (NT_STATUS_IS_OK(expected_status)) {
+		test_AsyncClosePrinter_byhandle(tctx, NULL, p, &server_handle);
+	}
+
+	talloc_free(p);
+
+	return true;
+}
+
+static bool test_object_uuid(struct torture_context *tctx,
+			     void *private_data)
+{
+	struct GUID object_uuid;
+
+	torture_assert(tctx,
+		test_object_one_uuid(tctx, NULL,
+			NT_STATUS_RPC_NOT_RPC_ERROR,
+			DCERPC_NCA_S_UNSUPPORTED_TYPE),
+		"failed to test NULL object uuid");
+
+	object_uuid = GUID_zero();
+	torture_assert(tctx,
+		test_object_one_uuid(tctx, &object_uuid,
+			NT_STATUS_RPC_NOT_RPC_ERROR,
+			DCERPC_NCA_S_UNSUPPORTED_TYPE),
+		"failed to test zeroed object uuid");
+
+	object_uuid = GUID_random();
+	torture_assert(tctx,
+		test_object_one_uuid(tctx, &object_uuid,
+			NT_STATUS_RPC_NOT_RPC_ERROR,
+			DCERPC_NCA_S_UNSUPPORTED_TYPE),
+		"failed to test random object uuid");
+
+	GUID_from_string(IREMOTEWINSPOOL_OBJECT_GUID, &object_uuid);
+	torture_assert(tctx,
+		test_object_one_uuid(tctx, &object_uuid,
+			NT_STATUS_OK,
+			0),
+		"failed to test IREMOTEWINSPOOL_OBJECT_GUID");
+
+	torture_assert(tctx,
+		test_object_one_uuid(tctx, &ndr_table_spoolss.syntax_id.uuid,
+			NT_STATUS_RPC_NOT_RPC_ERROR,
+			DCERPC_NCA_S_UNSUPPORTED_TYPE),
+		"failed to test spoolss interface uuid");
+
+	torture_assert(tctx,
+		test_object_one_uuid(tctx, &ndr_table_iremotewinspool.syntax_id.uuid,
+			NT_STATUS_RPC_NOT_RPC_ERROR,
+			DCERPC_NCA_S_UNSUPPORTED_TYPE),
+		"failed to test iremotewinspool interface uuid");
+
+	return true;
+}
+
+static bool test_setup_binding_handle(struct torture_context *tctx,
+				      struct GUID object_uuid,
+				      enum dcerpc_transport_t transport,
+				      const struct ndr_interface_table *table,
+				      struct dcerpc_pipe **p)
+{
+	struct dcerpc_binding *binding;
+
+	torture_assert_ntstatus_ok(tctx,
+		torture_rpc_binding(tctx, &binding),
+		"failed to retrieve torture binding");
+
+	torture_assert_ntstatus_ok(tctx,
+		dcerpc_binding_set_object(binding, object_uuid),
+		"failed to set object_uuid");
+
+	torture_assert_ntstatus_ok(tctx,
+		dcerpc_binding_set_transport(binding, transport),
+		"failed to set transport");
+
+	torture_assert_ntstatus_ok(tctx,
+		torture_rpc_connection_with_binding(tctx, binding, p, table),
+		"Error connecting to server");
+
+	return true;
+}
+
+static enum ndr_err_code ndr_push_inout_blob(DATA_BLOB *blob,
+					     TALLOC_CTX *mem_ctx,
+					     ndr_flags_type flags,
+					     const void *p,
+					     ndr_push_flags_fn_t fn)
+{
+	struct ndr_push *ndr;
+	ndr = ndr_push_init_ctx(mem_ctx);
+	NDR_ERR_HAVE_NO_MEMORY(ndr);
+
+	NDR_CHECK_FREE(fn(ndr, flags, p));
+
+	*blob = ndr_push_blob(ndr);
+	talloc_steal(mem_ctx, blob->data);
+	talloc_free(ndr);
+
+	return NDR_ERR_SUCCESS;
+}
+
+static bool test_compare_spoolss(struct torture_context *tctx,
+				 void *private_data)
+{
+	DATA_BLOB reply_iremotewinspool, reply_spoolss, request_spoolss;
+	struct dcerpc_pipe *iremotewinspool_pipe, *spoolss_pipe;
+	struct GUID object_uuid;
+	uint32_t out_flags;
+	NTSTATUS status;
+	struct spoolss_EnumPrinters r;
+	DATA_BLOB blob;
+
+	/* setup two dcerpc pipes */
+
+	torture_assert_ntstatus_ok(tctx,
+		GUID_from_string(IREMOTEWINSPOOL_OBJECT_GUID, &object_uuid),
+		"failed to parse GUID");
+
+	torture_assert(tctx,
+		test_setup_binding_handle(tctx, object_uuid, NCACN_IP_TCP,
+			&ndr_table_iremotewinspool, &iremotewinspool_pipe),
+		"failed to setup binding handle");
+
+	torture_assert(tctx,
+		test_setup_binding_handle(tctx, GUID_zero(), NCACN_NP,
+			&ndr_table_spoolss, &spoolss_pipe),
+		"failed to setup binding handle");
+
+
+	/* create a spoolss enumprinters request */
+
+	ZERO_STRUCT(r);
+
+	blob = data_blob_talloc_zero(tctx, 0x1000);
+
+	r.in.flags	= PRINTER_ENUM_LOCAL;
+	r.in.server	= talloc_asprintf(tctx, "\\\\%s",
+				dcerpc_server_name(spoolss_pipe));
+	r.in.level	= 1;
+	r.in.buffer	= &blob;
+	r.in.offered	= blob.length;
+
+	torture_assert_ndr_success(tctx,
+		ndr_push_inout_blob(&request_spoolss, tctx, NDR_IN | NDR_SET_VALUES, &r,
+			(ndr_push_flags_fn_t)ndr_push_spoolss_EnumPrinters),
+		"failed to push EnumPrinters request");
+
+	/* send same request to both endpoints */
+
+	status = dcerpc_binding_handle_raw_call(iremotewinspool_pipe->binding_handle,
+						NULL,
+						NDR_WINSPOOL_ASYNCENUMPRINTERS,
+						0, /* in_flags */
+						request_spoolss.data,
+						request_spoolss.length,
+						tctx,
+						&reply_iremotewinspool.data,
+						&reply_iremotewinspool.length,
+						&out_flags);
+	torture_assert_ntstatus_ok(tctx, status, "dcerpc_binding_handle_raw_call failed");
+
+	status = dcerpc_binding_handle_raw_call(spoolss_pipe->binding_handle,
+						NULL,
+						NDR_SPOOLSS_ENUMPRINTERS,
+						0, /* in_flags */
+						request_spoolss.data,
+						request_spoolss.length,
+						tctx,
+						&reply_spoolss.data,
+						&reply_spoolss.length,
+						&out_flags);
+	torture_assert_ntstatus_ok(tctx, status, "dcerpc_binding_handle_raw_call failed");
+
+	torture_assert_data_blob_equal(tctx,
+		reply_iremotewinspool,
+		reply_spoolss,
+		"unexpected difference in replies from spoolss and iremotewinspool servers");
+
+	talloc_free(iremotewinspool_pipe);
+	talloc_free(spoolss_pipe);
+
+	return true;
+}
+
 struct torture_suite *torture_rpc_iremotewinspool(TALLOC_CTX *mem_ctx)
 {
 	struct torture_suite *suite = torture_suite_create(mem_ctx, "iremotewinspool");
@@ -1060,5 +1082,9 @@ struct torture_suite *torture_rpc_iremotewinspool(TALLOC_CTX *mem_ctx)
 
 	torture_tcase_add_simple_test(tcase, "OpenPrinter", test_OpenPrinter);
 
+	tcase = torture_suite_add_tcase(suite, "protocol");
+	torture_tcase_add_simple_test(tcase, "object_uuid", test_object_uuid);
+	torture_tcase_add_simple_test(tcase, "compare_spoolss", test_compare_spoolss);
+
 	return suite;
 }
